@@ -16,9 +16,9 @@
 
 #include "sslerrordialog.h"
 #include "proxyauthhandler.h"
-#include "common/asserts.h"
 #include "creds/credentialsfactory.h"
 #include "creds/abstractcredentials.h"
+#include "creds/keychainchunk.h"
 #include "libsync/clientsideencryption.h"
 #include "libsync/configfile.h"
 #include "libsync/cookiejar.h"
@@ -29,6 +29,9 @@
 #include <QNetworkAccessManager>
 #include <QMessageBox>
 #include <QPushButton>
+#include <type_traits>
+
+#include <qt6keychain/keychain.h>
 
 namespace {
 constexpr auto urlC = "url";
@@ -46,6 +49,16 @@ constexpr auto serverVersionC = "serverVersion";
 constexpr auto serverColorC = "serverColor";
 constexpr auto serverTextColorC = "serverTextColor";
 constexpr auto skipE2eeMetadataChecksumValidationC = "skipE2eeMetadataChecksumValidation";
+constexpr auto networkProxySettingC = "networkProxySetting";
+constexpr auto networkProxyTypeC = "networkProxyType";
+constexpr auto networkProxyHostNameC = "networkProxyHostName";
+constexpr auto networkProxyPortC = "networkProxyPort";
+constexpr auto networkProxyNeedsAuthC = "networkProxyNeedsAuth";
+constexpr auto networkProxyUserC = "networkProxyUser";
+constexpr auto networkUploadLimitSettingC = "networkUploadLimitSetting";
+constexpr auto networkDownloadLimitSettingC = "networkDownloadLimitSetting";
+constexpr auto networkUploadLimitC = "networkUploadLimit";
+constexpr auto networkDownloadLimitC = "networkDownloadLimit";
 constexpr auto generalC = "General";
 
 constexpr auto dummyAuthTypeC = "dummy";
@@ -55,12 +68,16 @@ constexpr auto shibbolethAuthTypeC = "shibboleth";
 constexpr auto httpAuthPrefix = "http_";
 constexpr auto webflowAuthPrefix = "webflow_";
 
+constexpr auto networkProxyPasswordKeychainKeySuffixC = "_proxy_password";
+
 constexpr auto legacyRelativeConfigLocationC = "/ownCloud/owncloud.cfg";
 constexpr auto legacyCfgFileNameC = "owncloud.cfg";
 
 // The maximum versions that this client can read
 constexpr auto maxAccountsVersion = 2;
 constexpr auto maxAccountVersion = 1;
+
+constexpr auto serverHasValidSubscriptionC = "serverHasValidSubscription";
 }
 
 
@@ -106,7 +123,7 @@ AccountManager::AccountsRestoreResult AccountManager::restore(const bool alsoRes
             if (const auto acc = loadAccountHelper(*settings)) {
                 acc->_id = accountId;
                 const auto accState = new AccountState(acc);
-                const auto jar = qobject_cast<CookieJar*>(acc->_am->cookieJar());
+                const auto jar = qobject_cast<CookieJar*>(acc->_networkAccessManager->cookieJar());
                 Q_ASSERT(jar);
                 if (jar) {
                     jar->restore(acc->cookieJarPath());
@@ -288,12 +305,12 @@ void AccountManager::save(bool saveCredentials)
     qCInfo(lcAccountManager) << "Saved all account settings, status:" << settings->status();
 }
 
-void AccountManager::saveAccount(Account *a)
+void AccountManager::saveAccount(Account *newAccountData)
 {
-    qCDebug(lcAccountManager) << "Saving account" << a->url().toString();
+    qCDebug(lcAccountManager) << "Saving account" << newAccountData->url().toString();
     const auto settings = ConfigFile::settingsWithGroup(QLatin1String(accountsC));
-    settings->beginGroup(a->id());
-    saveAccountHelper(a, *settings, false); // don't save credentials they might not have been loaded yet
+    settings->beginGroup(newAccountData->id());
+    saveAccountHelper(newAccountData, *settings, false); // don't save credentials they might not have been loaded yet
     settings->endGroup();
 
     settings->sync();
@@ -311,48 +328,93 @@ void AccountManager::saveAccountState(AccountState *a)
     qCDebug(lcAccountManager) << "Saved account state settings, status:" << settings->status();
 }
 
-void AccountManager::saveAccountHelper(Account *acc, QSettings &settings, bool saveCredentials)
+void AccountManager::saveAccountHelper(Account *account, QSettings &settings, bool saveCredentials)
 {
     qCDebug(lcAccountManager) << "Saving settings to" << settings.fileName();
     settings.setValue(QLatin1String(versionC), maxAccountVersion);
-    settings.setValue(QLatin1String(urlC), acc->_url.toString());
-    settings.setValue(QLatin1String(davUserC), acc->_davUser);
-    settings.setValue(QLatin1String(displayNameC), acc->_displayName);
-    settings.setValue(QLatin1String(serverVersionC), acc->_serverVersion);
-    settings.setValue(QLatin1String(serverColorC), acc->_serverColor);
-    settings.setValue(QLatin1String(serverTextColorC), acc->_serverTextColor);
-    if (!acc->_skipE2eeMetadataChecksumValidation) {
+    settings.setValue(QLatin1String(urlC), account->_url.toString());
+    settings.setValue(QLatin1String(davUserC), account->_davUser);
+    settings.setValue(QLatin1String(displayNameC), account->davDisplayName());
+    settings.setValue(QLatin1String(serverVersionC), account->_serverVersion);
+    settings.setValue(QLatin1String(serverColorC), account->_serverColor);
+    settings.setValue(QLatin1String(serverTextColorC), account->_serverTextColor);
+    settings.setValue(QLatin1String(serverHasValidSubscriptionC), account->serverHasValidSubscription());
+
+    if (!account->_skipE2eeMetadataChecksumValidation) {
         settings.remove(QLatin1String(skipE2eeMetadataChecksumValidationC));
     } else {
-        settings.setValue(QLatin1String(skipE2eeMetadataChecksumValidationC), acc->_skipE2eeMetadataChecksumValidation);
+        settings.setValue(QLatin1String(skipE2eeMetadataChecksumValidationC), account->_skipE2eeMetadataChecksumValidation);
+    }
+    settings.setValue(networkProxySettingC, static_cast<std::underlying_type_t<Account::AccountNetworkProxySetting>>(account->networkProxySetting()));
+    settings.setValue(networkProxyTypeC, account->proxyType());
+    settings.setValue(networkProxyHostNameC, account->proxyHostName());
+    settings.setValue(networkProxyPortC, account->proxyPort());
+    settings.setValue(networkProxyNeedsAuthC, account->proxyNeedsAuth());
+    settings.setValue(networkProxyUserC, account->proxyUser());
+    settings.setValue(networkUploadLimitSettingC, static_cast<std::underlying_type_t<Account::AccountNetworkTransferLimitSetting>>(account->uploadLimitSetting()));
+    settings.setValue(networkDownloadLimitSettingC, static_cast<std::underlying_type_t<Account::AccountNetworkTransferLimitSetting>>(account->downloadLimitSetting()));
+    settings.setValue(networkUploadLimitC, account->uploadLimit());
+    settings.setValue(networkDownloadLimitC, account->downloadLimit());
+
+    const auto proxyPasswordKey = QString(account->userIdAtHostWithPort() + networkProxyPasswordKeychainKeySuffixC);
+    if (const auto proxyPassword = account->proxyPassword(); proxyPassword.isEmpty()) {
+        const auto job = new QKeychain::DeletePasswordJob(Theme::instance()->appName(), this);
+        job->setKey(proxyPasswordKey);
+        connect(job, &QKeychain::Job::finished, this, [](const QKeychain::Job *const incomingJob) {
+            if (incomingJob->error() == QKeychain::NoError) {
+                qCInfo(lcAccountManager) << "Deleted proxy password from keychain";
+            } else if (incomingJob->error() == QKeychain::EntryNotFound) {
+                qCDebug(lcAccountManager) << "Proxy password not found in keychain, can't delete";
+            } else {
+                qCWarning(lcAccountManager) << "Failed to delete proxy password to keychain" << incomingJob->errorString();
+            }
+        });
+        job->start();
+    } else {
+        const auto job = new QKeychain::WritePasswordJob(Theme::instance()->appName(), this);
+        job->setKey(proxyPasswordKey);
+        job->setBinaryData(proxyPassword.toUtf8());
+        connect(job, &QKeychain::Job::finished, this, [](const QKeychain::Job *const incomingJob) {
+            if (incomingJob->error() == QKeychain::NoError) {
+                qCInfo(lcAccountManager) << "Saved proxy password to keychain";
+            } else {
+                qCWarning(lcAccountManager) << "Failed to save proxy password to keychain" << incomingJob->errorString();
+            }
+        });
+        job->start();
     }
 
-    if (acc->_credentials) {
+    if (account->_credentials) {
         if (saveCredentials) {
             // Only persist the credentials if the parameter is set, on migration from 1.8.x
             // we want to save the accounts but not overwrite the credentials
             // (This is easier than asynchronously fetching the credentials from keychain and then
             // re-persisting them)
-            acc->_credentials->persist();
+            account->_credentials->persist();
         }
 
-        const auto settingsMapKeys = acc->_settingsMap.keys();
+        const auto settingsMapKeys = account->_settingsMap.keys();
         for (const auto &key : settingsMapKeys) {
-            settings.setValue(key, acc->_settingsMap.value(key));
+            if (!account->_settingsMap.value(key).isValid()) {
+                continue;
+            }
+
+            settings.setValue(key, account->_settingsMap.value(key));
         }
-        settings.setValue(QLatin1String(authTypeC), acc->_credentials->authType());
+        settings.setValue(QLatin1String(authTypeC), account->_credentials->authType());
 
         // HACK: Save http_user also as user
-        if (acc->_settingsMap.contains(httpUserC)) {
-            settings.setValue(userC, acc->_settingsMap.value(httpUserC));
+        const auto settingsMap = account->_settingsMap;
+        if (settingsMap.contains(httpUserC) && settingsMap.value(httpUserC).isValid()) {
+            settings.setValue(userC, settingsMap.value(httpUserC));
         }
     }
 
     // Save accepted certificates.
     settings.beginGroup(QLatin1String(generalC));
-    qCInfo(lcAccountManager) << "Saving " << acc->approvedCerts().count() << " unknown certs.";
+    qCInfo(lcAccountManager) << "Saving " << account->approvedCerts().count() << " unknown certs.";
     QByteArray certs;
-    const auto approvedCerts = acc->approvedCerts();
+    const auto approvedCerts = account->approvedCerts();
     for (const auto &cert : approvedCerts) {
         certs += cert.toPem() + '\n';
     }
@@ -362,13 +424,13 @@ void AccountManager::saveAccountHelper(Account *acc, QSettings &settings, bool s
     settings.endGroup();
 
     // Save cookies.
-    if (acc->_am) {
-        auto *jar = qobject_cast<CookieJar *>(acc->_am->cookieJar());
+    if (account->_networkAccessManager) {
+        const auto jar = qobject_cast<CookieJar *>(account->_networkAccessManager->cookieJar());
         if (jar) {
-            qCInfo(lcAccountManager) << "Saving cookies." << acc->cookieJarPath();
-            if (!jar->save(acc->cookieJarPath()))
+            qCInfo(lcAccountManager) << "Saving cookies." << account->cookieJarPath();
+            if (!jar->save(account->cookieJarPath()))
             {
-                qCWarning(lcAccountManager) << "Failed to save cookies to" << acc->cookieJarPath();
+                qCWarning(lcAccountManager) << "Failed to save cookies to" << account->cookieJarPath();
             }
         }
     }
@@ -401,7 +463,8 @@ AccountPtr AccountManager::loadAccountHelper(QSettings &settings)
 
     const auto overrideUrl = Theme::instance()->overrideServerUrl();
     const auto forceAuth = Theme::instance()->forceConfigAuthType();
-    if (!forceAuth.isEmpty() && !overrideUrl.isEmpty()) {
+    const auto multipleOverrideServers = Theme::instance()->multipleOverrideServers();
+    if (!forceAuth.isEmpty() && !overrideUrl.isEmpty() && !multipleOverrideServers) {
         // If forceAuth is set, this might also mean the overrideURL has changed.
         // See enterprise issues #1126
         acc->setUrl(overrideUrl);
@@ -435,7 +498,7 @@ AccountPtr AccountManager::loadAccountHelper(QSettings &settings)
     acc->_davUser = settings.value(QLatin1String(davUserC)).toString();
 
     acc->_settingsMap.insert(QLatin1String(userC), settings.value(userC));
-    acc->_displayName = settings.value(QLatin1String(displayNameC), "").toString();
+    acc->setDavDisplayName(settings.value(QLatin1String(displayNameC), "").toString());
     const QString authTypePrefix = authType + "_";
     const auto settingsChildKeys = settings.childKeys();
     for (const auto &key : settingsChildKeys) {
@@ -447,6 +510,41 @@ AccountPtr AccountManager::loadAccountHelper(QSettings &settings)
 
     acc->setCredentials(CredentialsFactory::create(authType));
 
+    acc->setNetworkProxySetting(settings.value(networkProxySettingC).value<Account::AccountNetworkProxySetting>());
+    acc->setProxyType(settings.value(networkProxyTypeC).value<QNetworkProxy::ProxyType>());
+    acc->setProxyHostName(settings.value(networkProxyHostNameC).toString());
+    acc->setProxyPort(settings.value(networkProxyPortC).toInt());
+    acc->setProxyNeedsAuth(settings.value(networkProxyNeedsAuthC).toBool());
+    acc->setProxyUser(settings.value(networkProxyUserC).toString());
+    acc->setUploadLimitSetting(
+        settings.value(
+            networkUploadLimitSettingC,
+            QVariant::fromValue(Account::AccountNetworkTransferLimitSetting::GlobalLimit)
+        ).value<Account::AccountNetworkTransferLimitSetting>());
+    acc->setDownloadLimitSetting(
+        settings.value(
+            networkDownloadLimitSettingC,
+            QVariant::fromValue(Account::AccountNetworkTransferLimitSetting::GlobalLimit)
+        ).value<Account::AccountNetworkTransferLimitSetting>());
+    acc->setUploadLimit(settings.value(networkUploadLimitC).toInt());
+    acc->setDownloadLimit(settings.value(networkDownloadLimitC).toInt());
+
+    const auto proxyPasswordKey = QString(acc->userIdAtHostWithPort() + networkProxyPasswordKeychainKeySuffixC);
+    const auto job = new QKeychain::ReadPasswordJob(Theme::instance()->appName(), this);
+    job->setKey(proxyPasswordKey);
+    connect(job, &QKeychain::Job::finished, this, [acc](const QKeychain::Job *const incomingJob) {
+        const auto incomingReadJob = qobject_cast<const QKeychain::ReadPasswordJob *>(incomingJob);
+        if (incomingReadJob->error() == QKeychain::NoError) {
+            qCInfo(lcAccountManager) << "Read proxy password to keychain for" << acc->userIdAtHostWithPort();
+            const auto passwordData = incomingReadJob->binaryData();
+            const auto password = QString::fromUtf8(passwordData);
+            acc->setProxyPassword(password);
+        } else {
+            qCWarning(lcAccountManager) << "Failed to read proxy password to keychain" << incomingJob->errorString();
+        }
+    });
+    job->start();
+
     // now the server cert, it is in the general group
     settings.beginGroup(QLatin1String(generalC));
     const auto certs = QSslCertificate::fromData(settings.value(caCertsKeyC).toByteArray());
@@ -515,10 +613,30 @@ void AccountManager::deleteAccount(OCC::AccountState *account)
 
     account->account()->deleteAppToken();
 
+    // clean up config from subscriptions if the account removed was the only with valid subscription
+    if (account->account()->serverHasValidSubscription()) {
+        updateServerHasValidSubscriptionConfig();
+    }
+
     emit accountSyncConnectionRemoved(account);
     emit accountRemoved(account);
 }
 
+void AccountManager::updateServerHasValidSubscriptionConfig()
+{
+    auto serverHasValidSubscription = false;
+    for (const auto &account : _accounts) {
+        if (!account->account()->serverHasValidSubscription()) {
+            continue;
+        }
+
+        serverHasValidSubscription = true;
+        break;
+    }
+
+    ConfigFile().setServerHasValidSubscription(serverHasValidSubscription);
+}
+
 AccountPtr AccountManager::createAccount()
 {
     const auto acc = Account::create();
@@ -574,10 +692,17 @@ void AccountManager::addAccountState(AccountState *const accountState)
     Q_ASSERT(accountState->account());
 
     QObject::connect(accountState->account().data(), &Account::wantsAccountSaved, this, &AccountManager::saveAccount);
+    QObject::connect(accountState->account().data(), &Account::capabilitiesChanged, this, &AccountManager::capabilitiesChanged);
 
     AccountStatePtr ptr(accountState);
     _accounts << ptr;
     ptr->trySignIn();
+
+    // update config subscriptions if the account added is the only with valid subscription
+    if (accountState->account()->serverHasValidSubscription() && !ConfigFile().serverHasValidSubscription()) {
+        updateServerHasValidSubscriptionConfig();
+    }
+
     emit accountAdded(accountState);
 }
 
