@@ -2,6 +2,15 @@
 
 #include "creds/httpcredentials.h"
 #include "creds/keychainchunk.h"
+#include "accessmanager.h"
+#include "account.h"
+#include "configfile.h"
+#include "theme.h"
+#ifdef WITH_WEBENGINE
+#include "wizard/webview.h"
+#endif // WITH_WEBENGINE
+#include "webflowcredentialsdialog.h"
+#include "networkjobs.h"
 
 #include <QAuthenticator>
 #include <QNetworkAccessManager>
@@ -12,15 +21,6 @@
 #include <QVBoxLayout>
 #include <QLabel>
 
-#include "accessmanager.h"
-#include "account.h"
-#include "configfile.h"
-#include "theme.h"
-#ifdef WITH_WEBENGINE
-#include "wizard/webview.h"
-#endif // WITH_WEBENGINE
-#include "webflowcredentialsdialog.h"
-
 using namespace QKeychain;
 
 namespace OCC {
@@ -190,7 +190,6 @@ void WebFlowCredentials::slotAskFromUserCredentialsProvided(const QString &user,
     emit asked();
 
     _askDialog->close();
-    _askDialog->deleteLater();
     _askDialog = nullptr;
 }
 
@@ -356,7 +355,7 @@ void WebFlowCredentials::forgetSensitiveData() {
 
     _account->deleteAppPassword();
 
-    const QString kck = keychainKey(_account->url().toString(), _user, _account->id());
+    const auto kck = keychainKey(_account->url().toString(), _user, _account->id());
     if (kck.isEmpty()) {
         qCDebug(lcWebFlowCredentials()) << "InvalidateToken: User is empty, bailing out!";
         return;
