@@ -35,8 +35,11 @@
 #include <QMessageBox>
 #include <QDebug>
 #include <QQuickStyle>
+#include <QStyle>
+#include <QStyleFactory>
 #include <QQuickWindow>
 #include <QSurfaceFormat>
+#include <QOperatingSystemVersion>
 
 using namespace OCC;
 
@@ -52,19 +55,15 @@ void warnSystray()
 
 int main(int argc, char **argv)
 {
-    qputenv("QTWEBENGINE_CHROMIUM_FLAGS", "--disable-gpu --no-sandbox");
-    QCoreApplication::setAttribute(Qt::AA_UseOpenGLES);
-
 #ifdef Q_OS_WIN
     SetDllDirectory(L"");
+    qputenv("QML_IMPORT_PATH", (QDir::currentPath() + QStringLiteral("/qml")).toLatin1());
 #endif
+
     Q_INIT_RESOURCE(resources);
     Q_INIT_RESOURCE(theme);
 
     // OpenSSL 1.1.0: No explicit initialisation or de-initialisation is necessary.
-
-    QCoreApplication::setAttribute(Qt::AA_UseHighDpiPixmaps, true);
-    QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling, true);
 #ifdef Q_OS_MAC
     Mac::CocoaInitializer cocoaInit; // RIIA
 #endif
@@ -74,20 +73,26 @@ int main(int argc, char **argv)
     QSurfaceFormat::setDefaultFormat(surfaceFormat);
 
     QQuickWindow::setTextRenderType(QQuickWindow::NativeTextRendering);
-    QQuickStyle::setStyle(QStringLiteral("Fusion"));
+
+    auto style = QStringLiteral("Fusion");
+
+#if defined Q_OS_MAC
+    style = QStringLiteral("macOS");
+#elif defined Q_OS_WIN
+    style = QStringLiteral("Fusion");
+#endif
+
+    QQuickStyle::setStyle(style);
+    QQuickStyle::setFallbackStyle(QStringLiteral("Fusion"));
+
+#if defined Q_OS_WIN
+    if (QOperatingSystemVersion::current().version() < QOperatingSystemVersion::Windows11.version()) {
+        QApplication::setStyle(QStyleFactory::create("Fusion"));
+    }
+#endif
 
     OCC::Application app(argc, argv);
 
-#ifdef Q_OS_WIN
-    // The Windows style still has pixelated elements with Qt 5.6,
-    // it's recommended to use the Fusion style in this case, even
-    // though it looks slightly less native. Check here after the
-    // QApplication was constructed, but before any QWidget is
-    // constructed.
-    if (app.devicePixelRatio() > 1)
-        QApplication::setStyle(QStringLiteral("fusion"));
-#endif // Q_OS_WIN
-
 #ifndef Q_OS_WIN
     signal(SIGPIPE, SIG_IGN);
 #endif
