@@ -159,8 +159,13 @@ void FileInfo::remove(const QString &relativePath)
     const PathComponents pathComponents { relativePath };
     FileInfo *parent = findInvalidatingEtags(pathComponents.parentDirComponents());
     Q_ASSERT(parent);
-    parent->children.erase(std::find_if(parent->children.begin(), parent->children.end(),
-        [&pathComponents](const FileInfo &fi) { return fi.name == pathComponents.fileName(); }));
+    auto childrenIt = std::find_if(parent->children.begin(), parent->children.end(),
+                                   [&pathComponents](const FileInfo &fi) {
+                                       return fi.name == pathComponents.fileName();
+                                   });
+    if (childrenIt != parent->children.end()) {
+        parent->children.erase(childrenIt);
+    }
 }
 
 void FileInfo::insert(const QString &relativePath, qint64 size, char contentChar)
@@ -173,6 +178,7 @@ void FileInfo::setContents(const QString &relativePath, char contentChar)
     FileInfo *file = findInvalidatingEtags(relativePath);
     Q_ASSERT(file);
     file->contentChar = contentChar;
+    file->lastModified = QDateTime::currentDateTimeUtc();
 }
 
 void FileInfo::appendByte(const QString &relativePath)
@@ -217,6 +223,13 @@ void FileInfo::setModTimeKeepEtag(const QString &relativePath, const QDateTime &
     file->lastModified = modTime;
 }
 
+void FileInfo::setIsLivePhoto(const QString &relativePath, const bool isLivePhoto)
+{
+    const auto file = find(relativePath);
+    Q_ASSERT(file);
+    file->isLivePhoto = isLivePhoto;
+}
+
 void FileInfo::modifyLockState(const QString &relativePath, LockState lockState, int lockType, const QString &lockOwner, const QString &lockOwnerId, const QString &lockEditorId, quint64 lockTime, quint64 lockTimeout)
 {
     FileInfo *file = findInvalidatingEtags(relativePath);
@@ -405,6 +418,7 @@ FakePropfindReply::FakePropfindReply(FileInfo &remoteRootFileInfo, QNetworkAcces
         xml.writeTextElement(ncUri, QStringLiteral("lock-time"), QString::number(fileInfo.lockTime));
         xml.writeTextElement(ncUri, QStringLiteral("lock-timeout"), QString::number(fileInfo.lockTimeout));
         xml.writeTextElement(ncUri, QStringLiteral("is-encrypted"), fileInfo.isEncrypted ? QString::number(1) : QString::number(0));
+        xml.writeTextElement(ncUri, QStringLiteral("metadata-files-live-photo"), fileInfo.isLivePhoto ? QString::number(1) : QString::number(0));
         buffer.write(fileInfo.extraDavProperties);
         xml.writeEndElement(); // prop
         xml.writeTextElement(davUri, QStringLiteral("status"), QStringLiteral("HTTP/1.1 200 OK"));
@@ -487,7 +501,7 @@ FileInfo *FakePutReply::perform(FileInfo &remoteRootFileInfo, const QNetworkRequ
         fileInfo->contentChar = putPayload.at(0);
     } else {
         // Assume that the file is filled with the same character
-        fileInfo = remoteRootFileInfo.create(fileName, putPayload.size(), putPayload.at(0));
+        fileInfo = remoteRootFileInfo.create(fileName, putPayload.size(), putPayload.isEmpty() ? ' ' : putPayload.at(0));
     }
     fileInfo->lastModified = OCC::Utility::qDateTimeFromTime_t(request.rawHeader("X-OC-Mtime").toLongLong());
     remoteRootFileInfo.find(fileName, /*invalidateEtags=*/true);
@@ -1076,7 +1090,7 @@ QNetworkReply *FakeQNAM::createRequest(QNetworkAccessManager::Operation op, cons
         const bool isUpload = newRequest.url().path().startsWith(sUploadUrl.path());
         FileInfo &info = isUpload ? _uploadFileInfo : _remoteRootFileInfo;
 
-        auto verb = newRequest.attribute(QNetworkRequest::CustomVerbAttribute);
+        auto verb = newRequest.attribute(QNetworkRequest::CustomVerbAttribute).toString();
         if (verb == QLatin1String("PROPFIND")) {
             // Ignore outgoingData always returning something good enough, works for now.
             reply = new FakePropfindReply { info, op, newRequest, this };
@@ -1359,7 +1373,7 @@ FakeFileLockReply::FakeFileLockReply(FileInfo &remoteRootFileInfo,
                                      QObject *parent)
     : FakePropfindReply(remoteRootFileInfo, op, request, parent)
 {
-    const auto verb = request.attribute(QNetworkRequest::CustomVerbAttribute);
+    const auto verb = request.attribute(QNetworkRequest::CustomVerbAttribute).toString();
 
     setRequest(request);
     setUrl(request.url());
